package dev.fyloz.trial.colorrecipesexplorer.service

import com.nhaarman.mockitokotlin2.doReturn
import com.nhaarman.mockitokotlin2.mock
import com.nhaarman.mockitokotlin2.spy
import com.nhaarman.mockitokotlin2.whenever
import dev.fyloz.trial.colorrecipesexplorer.model.*
import dev.fyloz.trial.colorrecipesexplorer.repository.RecipeRepository
import org.junit.jupiter.api.Nested
import org.junit.jupiter.api.Test
import kotlin.test.assertEquals
import kotlin.test.assertFalse
import kotlin.test.assertTrue

class RecipeServiceTest :
    AbstractExternalModelServiceTest<Recipe, RecipeSaveDto, RecipeUpdateDto, RecipeService, RecipeRepository>() {
    override val repository: RecipeRepository = mock()
    private val companyService: CompanyService = mock()
    override val service: RecipeService = spy(RecipeServiceImpl(repository, companyService))

    private val company: Company = company(id = 0L)
    override val entity: Recipe = recipe(id = 0L, name = "recipe", company = company)
    override val anotherEntity: Recipe = recipe(id = 1L, name = "another recipe", company = company)
    override val entitySaveDto: RecipeSaveDto = spy(recipeSaveDto(name = entity.name, companyId = entity.company.id!!))
    override val entityUpdateDto: RecipeUpdateDto = spy(recipeUpdateDto(id = entity.id!!, name = entity.name))

    @Nested
    inner class ExistsByCompany {
        @Test
        fun `returns true when at least one recipe exists for the given company`() {
            whenever(repository.existsByCompany(company)).doReturn(true)

            val found = service.existsByCompany(company)

            assertTrue(found)
        }

        @Test
        fun `returns false when no recipe exists for the given company`() {
            whenever(repository.existsByCompany(company)).doReturn(false)

            val found = service.existsByCompany(company)

            assertFalse(found)
        }
    }

    @Nested
    inner class GetAllByCompany {
        @Test
        fun `returns the recipes with the given company`() {
            val companies = listOf(entity, anotherEntity)
            whenever(repository.findAllByCompany(company)).doReturn(companies)

            val found = service.getAllByCompany(company)

            assertEquals(companies, found)
        }
    }

    @Nested
    inner class SaveDto {
        @Test
        fun `calls and returns save() with the created entity`() {
            whenever(companyService.getById(company.id!!)).doReturn(company)
            saveDtoTest(entity, entitySaveDto, service)
        }
    }
}
